<?php
#
# Script         phpDebug
# Copyright      2017 Denny Schlesinger
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# Autor          Denny Schlesinger
#
# $Id:$
#

/*************************************************************************
This class is specially useful to debug live websites that have a bug triggered by a remote client request. The remote client is not aware of this code, it does not interrupt program execution.

phpDebug() logs
	$_SERVER
	$_GET
	$_POST
	$_COOKIE
	Variables (optional)
which are sent by email if using phpErrorReporter 1.1.0 or higher and to the active php error log file.

Variable can be specified in one of two ways:
1.- An array of 'name=>variable' pairs, or
2.- A list of variables
The advantage of the array is that you can tell what the displayed values belong to.

To use phpDebug() add one line of code just before the line triggering the bug (two if you need autoload). You can use multiple instances of the phpDebug(), the output will show which line of code called it.

// good code
	new phpDebug([ array('alpha'=>$alpha [ , more name=>value pairs ]) | 
				$alpha [ , more values ]]);
// buggy code OR trigger_error() function
// more code 



*************************************************************************/

class phpDebug {

	// Properties
	private $trace;
	private $server;
	private $get;
	private $post;
	private $cookie;
	private $vars;
	
	// Constructor
	public function __construct() {
		$this->trace();
		$this->vars(func_num_args(), func_get_args());
	} // __construct()

	// Destructor
	public function __destruct() {
		$notice = "\nTrace:\n{$this->trace}
				  Server:\n{$this->server}
				  Get:\n{$this->get}
				  Post:\n{$this->post}
				  Cookie:\n{$this->cookie}
				  Variables:\n{$this->vars}\n";
# comment out one of the following two lines
		// Use with phpErrorReporter 1.1.0 or higher
		// Sends notice by email and to php error log
       	noticeReporter($notice);
       	// Sends notice to php error log
#		trigger_error(notice, E_USER_NOTICE);
   } // __destruct()
	
	// Methods
	private function trace() {
		ob_start();
		debug_print_backtrace();
		$this->trace = ob_get_contents();
		ob_end_clean();	
	} // trace()
	
	private function vars($numVars, $variables) {
		$this->server = print_r($_SERVER, true);
		
		if(empty($_GET)) {
			$this->get = 'N/A';
		} else {
			$this->get = print_r($_GET, true);
		}
		
		if(empty($_POST)) {
			$this->post = 'N/A';
		} else {
			$this->post = print_r($_POST, true);
		}
		
		if(empty($_COOKIE)) {
			$this->cookie = 'N/A';
		} else {
			$this->cookie = print_r($_COOKIE, true);
		}

		if(!$numVars) {
			$this->vars = 'N/A';
		} elseif(is_array($variables[0])) {
			$this->vars  = count($variables[0])."\n";
			$this->vars .= print_r($variables[0], true);
			if($numVars > 1) {
				$this->vars .= "Additional phpDebug() arguments ignored\n";
			}
		} else {
			$this->vars  = "{$numVars}\n";
			$this->vars .= print_r($variables, true);
		}
	} // vars()

} // phpDebug {}

?>
